/*
Nome Artefato/Programa..: send.pgc
Empresa.................: UFRN-SINFO Universidade Federal do Rio Grande do Norte  
Autor(es)...............: Emerson Hermann (emersonhermann@gmail.com) (emerson@nfo.ufrn.br) 
Data Inicio ............: 07/01/2010
Data Atual..............: 01/02/2010
Versao..................: 0.01
Compilador/Interpretador: Postgres(ecpg) 4.5.0 / (ecpg)4.4.0 / GNU (gcc) 4.3.3 / GNU(gcc)4.2.4 / GNU(gcc)3.3.4
Sistemas Operacionais...: Linux
SGBD....................: PostgreSql 8.0, 8.1, 8.2, 8.3, 8.4 
Kernel..................: testado (2.6.27-7) (2.6.21-5) (2.4.29 com lib krb5-1.4.2-i486.22ttw.tgz) 
Finalidade..............: Processar e enviar logs do postgres
OBS.....................: SQL Embedded
*/
#include <stdio.h>
#include <stdlib.h>
//#include <process.h> 	// ativar para plataforma Windows 
#include <memory.h>
#include <string.h>
#include <errno.h> 	// tratamento de erros  
#include <strings.h>
#include <sys/types.h> 	// necessario para o daemonInit
#include <sys/stat.h> 	// necessario para o daemonInit
#include <fcntl.h> 	// necessario para o daemonInit
#include <unistd.h> 	// necessario no Unix/Linux para a funcao sleep
#include <time.h> 	// funcoes time_t, time, gmtime, localtime



EXEC SQL BEGIN DECLARE SECTION;

  //const char databasename [140] = "tcp:postgresql://10.0.2.15/test";
 
  // Conexao localhost
  const char databasename [140] = "tcp:postgresql://localhost:5432/test";
 
  const char userx[10] = "postgres";
  const char passx[10] = "postgres";
  int id_pesagem;
  //struct varchar_var  { int len; char arr[14]; } codigo_volume;
  char codigo_volume [14];
  float peso_bruto;
  long lCountDb;
  

EXEC SQL END DECLARE SECTION;

/* Incluso da Estrutura p/Tratamento de Erros - SQLCA */
EXEC SQL INCLUDE sqlca;

long    error=0L;


long printMsgError (void);

/***********************************************************
 *  Function connectDataBase ==> Conecta ao DB Postgres  *
 ***********************************************************/
int connectDataBase(void)
{
  /* Conecta ao Database */

  EXEC SQL CONNECT TO :databasename AS myconnect USER :userx IDENTIFIED BY :passx; 
  
  //EXEC SQL CONNECT TO "tcp:postgresql://10.0.2.15:5432/test" AS myconnect USER pgsql IDENTIFIED BY pgsql; 
  // Conexao localhost
  //EXEC SQL CONNECT TO :databasename ; 
  error = printMsgError();

  if (error) 
  {
    printf("\nSQL ERROR> Nao foi possivel conectar ao Postgres Server!\n");
    exit(1);
  }
  
}

/*************************************************************
 *Function disconnectDataBase ==> Desconecta do DB Postgres*
 *************************************************************/
void disconnectDataBase(void)
{
  long lRet;

  /* Desconecta do Database atualmente conectado  */
  EXEC SQL WHENEVER SQLERROR CONTINUE;
  EXEC SQL DISCONNECT CURRENT;
  printf ("SQL> Desconectando o Postgres...\n");

  if (SQLCODE)
  {
    printf("\nSQL> Detectado Erro na Desconexao no Postgres!\n");
    lRet = printMsgError();
    EXEC SQL ROLLBACK;
    exit (0);
  }

  printf ("SQL> Desconexao do Postgres Realizada com Sucesso!\n");
  exit(1);

}

/***************************************************************
 *  Function PrintMsgError ==> Imprime uma Mensagem de Erro    *
 *                             refletindo o SQLCODE retornado  *
 *			       pelo Postgres                 *
 ***************************************************************/
long printMsgError (void)
{
  if (SQLCODE)
  {
    printf ("\nSQL ERROR> Postgres Error Code = %d.\n", SQLCODE);
  }
  return (SQLCODE);
}

/***************************************************************
 *  Function execExport ==> SELECT *                           * 
 *                            FROM ddl                         *
 *                           WHERE exportado IS FALSE          *
 ***************************************************************/
int execExport (void)
{

  int        iFd;  		// ponteiro do arquivo de importacao (Unix/Linux)
  char       szBuffer[300]; 	// para armazenar strings de comandos dml no arquivo de importacao
  long       i; 		// controle de loop
  time_t     stHora_sist; 	// Para armazenar a data hora do sistema em quantidade de segundos 
  struct tm  *ST_Hora_exp; 	// Para armazenar o formato expandido da data 


  /*
    Para ver se existem tuplas nao exportados
  */
  EXEC SQL 
	  SELECT COUNT (*) INTO :lCountDb 
            FROM ddl;


  /* Declara um cursor para execucao da query.
   * Selecao de registros que nao foram exportados 
   */
  EXEC SQL DECLARE cs_tb CURSOR FOR 
                   SELECT banco,
		          usuario, 
			  maquina,
			  data_hora, 
			  log
                     FROM ddl
                    ORDER BY data_hora;

  
  /* Abre o cursor */
  EXEC SQL OPEN cs_tb;
  

  /* Atualiza registro(s) exportado(s) FALSE para TRUE.
   * Garantia que o(s) registro(s) ora transacionados
   * nao serao exportados novamente
   */

  
  EXEC SQL 
	  UPDATE ddl
	     SET exportado = TRUE
	   WHERE exportado IS FALSE;
  
  

  /* So por seguranca, para evitar gasto de processamento 
   * caso ocorra algum erro no processamento da(s) query(s)
   * evitando perder tempo com algo com problemas - desativado
   */
  /* 
  if (SQLCODE)
  {
    printMsgError();
    return (0);
  }
  */

  /* Caso a transacao seja efetuada sem 
   * problemas, sera efetuado um COMMIT e
   * geracao de arquivo de exportacao,
   * caso contrario sera efetuado ROLLBACK
   */
  

  if (SQLCODE = printMsgError())
  {
    EXEC SQL ROLLBACK;
    printf ("SQL ERROR> Erro na Executacao da Query...\n");
  }
  else 
  {
    // Testa se veio alguma coisa do banco 
    if (lCountDb != 0)
    {
      /* Obtendo a data e hora atual do sistema 
       * Para configurar timezone no Unix/Linux: 
       * export TZ="Brazil/East"
       * Para configurar timezone no Windows:
       * set TZ=GMT-3
       */
      stHora_sist = time(NULL); // total em segundos
      ST_Hora_exp = localtime (&stHora_sist); // obtendo a data expandida considerando o timezone      

      /* criando arquivo de importacao */
      sprintf (szBuffer, "/tmp/lgsql_%02d_%02d_%04d_%02d_%02d_%02d.sql",ST_Hora_exp->tm_mday,ST_Hora_exp->tm_mon+1,(ST_Hora_exp->tm_year-100)+2000,ST_Hora_exp->tm_hour,ST_Hora_exp->tm_min,ST_Hora_exp->tm_sec);

      /* Abre arquivo com a devida permissao no SO */
      iFd = open (szBuffer, O_CREAT | O_WRONLY, 0400);

      /* Abrindo transacao no arquivo a ser importado  */

      sprintf(szBuffer, "--PID %d\nCOMMIT;\nSET TRANSACTION READ WRITE ISOLATION LEVEL READ COMMITTED NAME 'lg_sql_transaction_import'\n\n",getpid());

      write(iFd, szBuffer, strlen (szBuffer)); 

      /* Variavel de controle de loop*/
      i=1;
      do 
      {

	/* Execucao do fetch no curso aberto anteriormente */      
        EXEC SQL FETCH NEXT FROM cs_tb INTO :id_pesagem, :codigo_volume, :peso_bruto;

	//printf("INFO> Registro Id: %d - %14s - %14.2f\n",id_pesagem, codigo_volume, peso_bruto);

        sprintf(szBuffer, "INSERT INTO sd3010 (D3_FILIAL, D3_TM, D3_COD, D3_UM, D3_QUANT, D3_PESOB, D3_TARA, D3_VOL, D3_HORA) \
		       	VALUES (%d,%14s,%7.2f);\n\n",id_pesagem,codigo_volume, peso_bruto);
        write(iFd, szBuffer, strlen (szBuffer)); 
	i++;

      }  
      while (i<=lCountDb);
     

      /* Efetuando COMMIT no arquivo a ser importado  */

      sprintf(szBuffer, "COMMIT;\nQUIT;\n");
      write(iFd, szBuffer, strlen (szBuffer)); 

      /* Fechando o arquivo, uma boa pratica */
      close(iFd);
 

    }
    EXEC SQL CLOSE cs_tb;

    EXEC SQL COMMIT;

    //printf ("INFO> Registro(s) processado(s)!\n");

  }

  return (0);
  
}
/***************************************************************
 *  Function daemonInit ==> Funcao para tramsformar processo   *
 *                           terminal em processo daemon       *
 *                           (Linux / Unix)                    *
 ***************************************************************/
int daemonInit (void)
{
  pid_t     iPid;
  long      iMaxFd;
  int       i;

  /* 1o. Passo - Duplica o processo usando fork. 
   * O processo pai eh encerrado.
   */

  if ((iPid = fork()) < 0)
    return -1;

  if (iPid != 0)
    exit(0);


  /* 2o. Passo - Chama a funcao setsid para criar uma nova sessao de processo, 
   * ficando o processo filho como lider da sessao e sem um terminal de controle 
   * associado ao processo 
   */

  setsid();

  /* 3o. Passo - Mudando para o diretorio raiz (root) ou para o diretorio 
   * proprio do daemon 
   */

  chdir ("/");

  /* 4o. Passo - Inicializa a mascara padrao de criacao (permissao) de arquivos  
   */

  umask (0);

  /* 5o. Passo - Fechando todos descritores existentes no sistema. Utiliza-se a informacao
   * de numero maximo de descritores configurado no sistema, obtido com a funcao sysconf.
   * Para outras opcoes da funcao sysconf, favor verificar o manual on-line do sistema
   */

  iMaxFd = sysconf (_SC_OPEN_MAX);
  for (i=0; i < iMaxFd; i++)
  {
    close (i);
  }

  return 0;
}


#if defined(UNIX) || defined(WINNT)
  void sighandler(sig)
  int sig;
  {
    disconnectDataBase ();
    exit(0);
  }
#endif


/*******************
 *  FUNCTION MAIN  *
 *******************/
int main (int argc, char *argv[])
{

  system("clear");
  printf ("**************************************************************\n");
  printf ("**         pgLogger - Processa logs do Postgres             **\n");
  printf ("**                   send - Versao 0.1                      **\n");  
  printf ("** -                 Desenvolvido por:                    - **\n"); 
  printf ("** - Emerson Hermann - Email: (emersonhermann@gmail.com)  - **\n"); 
  printf ("**                       Agradecimentos                     **\n"); 
  printf ("**                 Obrigado JESUS por tudo !!!!             **\n"); 
  printf ("**************************************************************\n\n");

  /* Adaptacoes futuras para Windows / Linux / Unix */

  #if defined(UNIX) || defined(WINNT)
    signal(SIGINT,(void (__cdecl *) (int))sighandler);
    signal(SIGTERM,(void (__cdecl *) (int))sighandler);
    signal(SIGSEGV,(void (__cdecl *) (int))sighandler);
  #ifndef UNIX
    signal(SIGQUIT,sighandler);
    signal(SIGBUS,sighandler);
    signal(SIGHUP,sighandler);
  #endif
  #endif

  /* Aqui faz a brincadeira de rodar em daemon */  


  if (daemonInit () < 0) 
  {
    perror (argv[0]);  	  
    exit (errno);		
  }
 

  
  /* Conectar ao Postgres */
  connectDataBase();

  /* Apenas para tratar o erro do SQL, continuar rodando caso encontre erro */
  EXEC SQL WHENEVER SQLERROR CONTINUE;

  while (1)
  {

	  
    // Processo de execucao da exportacao, geracao de querys e arquivos 	
    execExport();

    // Delay, tempo de intervalo de execucao em segundos 
    sleep(3);

    /* Apenas um teste */

    system("cat /proc/ide/hda/model >> /tmp/test.txt");


  }

  exit (0);

  /* Este artefato rodara eternamente em processo daemon (function daemonInit), 
   * nao sendo necessario fechar conexao, mas por precaucao caso seja dado um  
   * kill no processo, havera uma tentativa de fechamento da conexao
   * com o banco, muito embora ja exista algo para tratamento desta situacao. 
   */

  disconnectDataBase ();
}

